
package com.rjconsultores.ventaboletos.web.gui.controladores.catalogos;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Controller;
import org.zkoss.util.resource.Labels;
import org.zkoss.zk.ui.Component;
import org.zkoss.zk.ui.event.Event;
import org.zkoss.zk.ui.event.EventListener;
import org.zkoss.zul.Paging;

import com.rjconsultores.ventaboletos.entidad.CodigoBarraTrecho;
import com.rjconsultores.ventaboletos.entidad.Custom;
import com.rjconsultores.ventaboletos.entidad.Empresa;
import com.rjconsultores.ventaboletos.entidad.Parada;
import com.rjconsultores.ventaboletos.service.CustomService;
import com.rjconsultores.ventaboletos.service.EmpresaService;
import com.rjconsultores.ventaboletos.utilerias.CustomEnum;
import com.rjconsultores.ventaboletos.web.utilerias.MyComboboxEmpresa;
import com.rjconsultores.ventaboletos.web.utilerias.MyComboboxParada;
import com.rjconsultores.ventaboletos.web.utilerias.MyGenericForwardComposer;
import com.rjconsultores.ventaboletos.web.utilerias.MyListbox;
import com.rjconsultores.ventaboletos.web.utilerias.MyTextbox;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.HibernateSearchObject;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.PagedListWrapper;
import com.rjconsultores.ventaboletos.web.utilerias.render.RenderCodigoBarraTrecho;
import com.rjconsultores.ventaboletos.web.utilerias.spring.AppContext;

@Controller("busquedaCodigoBarraTrechoController")
@Scope("prototype")
public class BusquedaCodigoBarraTrechoController extends MyGenericForwardComposer {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	@Autowired
	private transient PagedListWrapper<CodigoBarraTrecho> plwCodigoBarraTrecho;
	private MyListbox codigoBarraTrechoList;
	private Paging pagingCodigoBarraTrecho;
	private MyComboboxEmpresa comboEmpresa;
	private MyComboboxParada cmbOrigen;
	private MyComboboxParada cmbDestino;
	private MyTextbox txtCodTerminal;
	private MyTextbox txtCodEmpresa;
	private MyTextbox txtCodRuta;
	
	private List<Empresa> lsEmpresa;
	
	@Autowired
	private EmpresaService empresaService;

	@Override
	public void doAfterCompose(Component comp) throws Exception {
		
		lsEmpresa = empresaService.obtenerTodos();
		
		super.doAfterCompose(comp);

		codigoBarraTrechoList.setItemRenderer(new RenderCodigoBarraTrecho());
		codigoBarraTrechoList.addEventListener("onDoubleClick", new EventListener() {

			@Override
			public void onEvent(Event event) throws Exception {
				CodigoBarraTrecho codigoBarraTrecho = (CodigoBarraTrecho) codigoBarraTrechoList.getSelected();
				verCodigoBarraTrecho(codigoBarraTrecho);
			}
		});

		refreshLista();
	}

	@SuppressWarnings({ "unchecked", "rawtypes" })
	private void verCodigoBarraTrecho(CodigoBarraTrecho codigoBarraTrecho) {
		if (codigoBarraTrecho == null) {
			return;
		}

		Map args = new HashMap();
		args.put("codigoBarraTrecho", codigoBarraTrecho);
		args.put("codigoBarraTrechoList", codigoBarraTrechoList);

		openWindow("/gui/catalogos/editarCodigoBarraTrecho.zul", Labels.getLabel("editarConfiguracaoCodBarrasTrechoController.title"), args,
				MODAL);
	}

	private void refreshLista() {
		HibernateSearchObject<CodigoBarraTrecho> codigoBarraTrechoBusqueda = new HibernateSearchObject<CodigoBarraTrecho>(
				CodigoBarraTrecho.class, pagingCodigoBarraTrecho.getPageSize());

		if (comboEmpresa.getSelectedItem() != null) {
			Empresa empresa = (Empresa) comboEmpresa.getSelectedItem().getValue();
			codigoBarraTrechoBusqueda.addFilterEqual("empresa.empresaId", empresa.getEmpresaId());
		}
		
		if (cmbOrigen.getSelectedItem() != null) {
			Parada origen = (Parada) cmbOrigen.getSelectedItem().getValue();
			codigoBarraTrechoBusqueda.addFilterEqual("origen.paradaId", origen.getParadaId());
		}
		
		if (cmbDestino.getSelectedItem() != null) {
			Parada destino = (Parada) cmbDestino.getSelectedItem().getValue();
			codigoBarraTrechoBusqueda.addFilterEqual("destino.paradaId", destino.getParadaId());
		}
		
		if (StringUtils.isNotBlank(txtCodTerminal.getText())) {
			codigoBarraTrechoBusqueda.addFilterLike("codTerminal",  txtCodTerminal.getText());
		}
		
		if (StringUtils.isNotBlank(txtCodEmpresa.getText())) {
			codigoBarraTrechoBusqueda.addFilterLike("codEmpresa",  txtCodEmpresa.getText());
		}
		
		if (StringUtils.isNotBlank(txtCodRuta.getText())) {
			codigoBarraTrechoBusqueda.addFilterLike("codRuta",  txtCodRuta.getText());
		}

		codigoBarraTrechoBusqueda.addFilterEqual(ACTIVO, Boolean.TRUE);
		
		codigoBarraTrechoBusqueda.addSortAsc("empresa");

		plwCodigoBarraTrecho.init(codigoBarraTrechoBusqueda, codigoBarraTrechoList, pagingCodigoBarraTrecho);

	}

	public void onClick$btnPesquisa(Event ev) {
		refreshLista();
	}

	public void onClick$btnRefresh(Event ev) {
		refreshLista();
	}

	public void onClick$btnNovo(Event ev) {
		verCodigoBarraTrecho(new CodigoBarraTrecho());
	}
	
	public void onClick$btnPadrao(Event ev) {
		CustomService customService = (CustomService) AppContext.getApplicationContext().getBean("customService");
		Custom custom = customService.buscar(CustomEnum.PADRAO_CODIGO_BARRA_TRECHO.getDescricao());
		
		if (custom == null) {
			return;
		}

		Map args = new HashMap();
		args.put("custom", custom);
		args.put("customList", new MyListbox());

		openWindow("/gui/catalogos/editarCustom.zul",
				Labels.getLabel("customController.window.title"), args, MODAL);
		
	}

	public PagedListWrapper<CodigoBarraTrecho> getPlwCodigoBarraTrecho() {
		return plwCodigoBarraTrecho;
	}

	public void setPlwCodigoBarraTrecho(PagedListWrapper<CodigoBarraTrecho> plwCodigoBarraTrecho) {
		this.plwCodigoBarraTrecho = plwCodigoBarraTrecho;
	}

	public MyListbox getCodigoBarraTrechoList() {
		return codigoBarraTrechoList;
	}

	public void setCodigoBarraTrechoList(MyListbox codigoBarraTrechoList) {
		this.codigoBarraTrechoList = codigoBarraTrechoList;
	}

	public Paging getPagingCodigoBarraTrecho() {
		return pagingCodigoBarraTrecho;
	}

	public void setPagingCodigoBarraTrecho(Paging pagingCodigoBarraTrecho) {
		this.pagingCodigoBarraTrecho = pagingCodigoBarraTrecho;
	}

	public MyComboboxEmpresa getComboEmpresa() {
		return comboEmpresa;
	}

	public void setComboEmpresa(MyComboboxEmpresa comboEmpresa) {
		this.comboEmpresa = comboEmpresa;
	}

	public MyComboboxParada getCmbDestino() {
		return cmbDestino;
	}

	public void setCmbDestino(MyComboboxParada cmbDestino) {
		this.cmbDestino = cmbDestino;
	}

	public MyTextbox getTxtCodTerminal() {
		return txtCodTerminal;
	}

	public void setTxtCodTerminal(MyTextbox txtCodTerminal) {
		this.txtCodTerminal = txtCodTerminal;
	}

	public MyTextbox getTxtCodEmpresa() {
		return txtCodEmpresa;
	}

	public void setTxtCodEmpresa(MyTextbox txtCodEmpresa) {
		this.txtCodEmpresa = txtCodEmpresa;
	}

	public MyTextbox getTxtCodRuta() {
		return txtCodRuta;
	}

	public void setTxtCodRuta(MyTextbox txtCodRuta) {
		this.txtCodRuta = txtCodRuta;
	}

	public List<Empresa> getLsEmpresa() {
		return lsEmpresa;
	}

	public void setLsEmpresa(List<Empresa> lsEmpresa) {
		this.lsEmpresa = lsEmpresa;
	}

	public EmpresaService getEmpresaService() {
		return empresaService;
	}

	public void setEmpresaService(EmpresaService empresaService) {
		this.empresaService = empresaService;
	}

	public MyComboboxParada getCmbOrigen() {
		return cmbOrigen;
	}

	public void setCmbOrigen(MyComboboxParada cmbOrigen) {
		this.cmbOrigen = cmbOrigen;
	}


}
