
package com.rjconsultores.ventaboletos.web.gui.controladores.catalogos;

import java.util.List;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Controller;
import org.zkoss.util.resource.Labels;
import org.zkoss.zhtml.Messagebox;
import org.zkoss.zk.ui.Component;
import org.zkoss.zk.ui.Executions;
import org.zkoss.zk.ui.event.Event;
import org.zkoss.zul.Button;

import com.rjconsultores.ventaboletos.entidad.CodigoBarraTrecho;
import com.rjconsultores.ventaboletos.entidad.Empresa;
import com.rjconsultores.ventaboletos.entidad.Parada;
import com.rjconsultores.ventaboletos.service.CodigoBarraTrechoService;
import com.rjconsultores.ventaboletos.service.EmpresaService;
import com.rjconsultores.ventaboletos.web.utilerias.MyComboboxEmpresa;
import com.rjconsultores.ventaboletos.web.utilerias.MyComboboxParada;
import com.rjconsultores.ventaboletos.web.utilerias.MyGenericForwardComposer;
import com.rjconsultores.ventaboletos.web.utilerias.MyListbox;
import com.rjconsultores.ventaboletos.web.utilerias.MyTextbox;

@Controller("editarCodigoBarraTrechoController")
@Scope("prototype")
public class EditarCodigoBarraTrechoController extends MyGenericForwardComposer {

	private static final long serialVersionUID = 1L;

	@Autowired
	private EmpresaService empresaService;

	@Autowired
	private CodigoBarraTrechoService codigoBarraTrechoService;

	private CodigoBarraTrecho codigoBarraTrecho;

	private MyListbox codigoBarraTrechoList;
	private List<Empresa> lsEmpresa;
	private List<Parada> lsParada;

	private MyComboboxEmpresa cmbEmpresa;
	private MyComboboxParada cmbOrigen;
	private MyComboboxParada cmbDestino;
	private MyTextbox txtCodTerminal;
	private MyTextbox txtCodEmpresa;
	private MyTextbox txtCodRuta;
	private Button btnApagar;

	private static Logger log = LogManager.getLogger(EditarCodigoBarraTrechoController.class);

	@Override
	public void doAfterCompose(Component comp) throws Exception {
		super.doAfterCompose(comp);

		codigoBarraTrecho = (CodigoBarraTrecho) Executions.getCurrent().getArg().get("codigoBarraTrecho");
		codigoBarraTrechoList = (MyListbox) Executions.getCurrent().getArg().get("codigoBarraTrechoList");

		if (codigoBarraTrecho != null && codigoBarraTrecho.getCodigoBarraId() != null) {
			cmbEmpresa.setText(codigoBarraTrecho.getEmpresa().getNombempresa());
			cmbOrigen.setComboItemByParada(codigoBarraTrecho.getOrigen());
			cmbDestino.setComboItemByParada(codigoBarraTrecho.getDestino());
			txtCodTerminal.setText(codigoBarraTrecho.getCodTerminal());
			txtCodEmpresa.setText(codigoBarraTrecho.getCodEmpresa());
			txtCodRuta.setText(codigoBarraTrecho.getCodRuta());
		} else {
			btnApagar.setVisible(false);
		}

	}

	public void onClick$btnSalvar(Event ev) throws InterruptedException {
		try {
			
			codigoBarraTrecho.setActivo(true);
			codigoBarraTrecho.setEmpresa((Empresa) cmbEmpresa.getSelectedItem().getValue());
			codigoBarraTrecho.setOrigen((Parada) cmbOrigen.getSelectedItem().getValue());
			codigoBarraTrecho.setDestino((Parada) cmbDestino.getSelectedItem().getValue());
			codigoBarraTrecho.setCodTerminal(txtCodTerminal.getText());
			codigoBarraTrecho.setCodEmpresa(txtCodEmpresa.getText());
			codigoBarraTrecho.setCodRuta(txtCodRuta.getText());

			codigoBarraTrechoList.removeItem(codigoBarraTrecho);

			if (codigoBarraTrecho.getCodigoBarraId() == null) {
				codigoBarraTrechoList.addItemNovo(codigoBarraTrechoService.suscribir(codigoBarraTrecho));
			} else {
				codigoBarraTrechoList.addItemNovo(codigoBarraTrechoService.actualizacion(codigoBarraTrecho));
			}
			
			closeWindow();
		} catch (Exception e) {
			log.error(e.getLocalizedMessage(), e);
		}

	}

	public void onClick$btnApagar(Event ev) throws InterruptedException {
		try {
			int resp = Messagebox.show(Labels.getLabel("editarConfiguracaoCodBarrasTrechoController.MSG.borrarPergunta"),
					Labels.getLabel("editarConfiguracaoCodBarrasTrechoController.title"), Messagebox.YES | Messagebox.NO,
					Messagebox.QUESTION);

			if (resp == Messagebox.YES) {

				codigoBarraTrechoService.borrar(codigoBarraTrecho);

				Messagebox.show(Labels.getLabel("editarConfiguracaoCodBarrasTrechoController.MSG.borrarOK"),
						Labels.getLabel("editarConfiguracaoCodBarrasTrechoController.title"), Messagebox.OK,
						Messagebox.INFORMATION);

				codigoBarraTrechoList.removeItem(codigoBarraTrecho);

				closeWindow();
			}
		} catch (Exception e) {
			log.error(e.getLocalizedMessage(), e);
		}

	}

	public EmpresaService getEmpresaService() {
		return empresaService;
	}

	public void setEmpresaService(EmpresaService empresaService) {
		this.empresaService = empresaService;
	}

	public CodigoBarraTrechoService getCodigoBarraTrechoService() {
		return codigoBarraTrechoService;
	}

	public void setCodigoBarraTrechoService(CodigoBarraTrechoService codigoBarraTrechoService) {
		this.codigoBarraTrechoService = codigoBarraTrechoService;
	}

	public CodigoBarraTrecho getCodigoBarraTrecho() {
		return codigoBarraTrecho;
	}

	public void setCodigoBarraTrecho(CodigoBarraTrecho codigoBarraTrecho) {
		this.codigoBarraTrecho = codigoBarraTrecho;
	}

	public MyListbox getCodigoBarraTrechoList() {
		return codigoBarraTrechoList;
	}

	public void setCodigoBarraTrechoList(MyListbox codigoBarraTrechoList) {
		this.codigoBarraTrechoList = codigoBarraTrechoList;
	}

	public List<Empresa> getLsEmpresa() {
		return lsEmpresa;
	}

	public void setLsEmpresa(List<Empresa> lsEmpresa) {
		this.lsEmpresa = lsEmpresa;
	}

	public List<Parada> getLsParada() {
		return lsParada;
	}

	public void setLsParada(List<Parada> lsParada) {
		this.lsParada = lsParada;
	}

	public MyComboboxEmpresa getCmbEmpresa() {
		return cmbEmpresa;
	}

	public void setCmbEmpresa(MyComboboxEmpresa cmbEmpresa) {
		this.cmbEmpresa = cmbEmpresa;
	}

	public MyComboboxParada getCmbOrigen() {
		return cmbOrigen;
	}

	public void setCmbOrigen(MyComboboxParada cmbOrigen) {
		this.cmbOrigen = cmbOrigen;
	}

	public MyComboboxParada getCmbDestino() {
		return cmbDestino;
	}

	public void setCmbDestino(MyComboboxParada cmbDestino) {
		this.cmbDestino = cmbDestino;
	}

	public MyTextbox getTxtCodTerminal() {
		return txtCodTerminal;
	}

	public void setTxtCodTerminal(MyTextbox txtCodTerminal) {
		this.txtCodTerminal = txtCodTerminal;
	}

	public MyTextbox getTxtCodEmpresa() {
		return txtCodEmpresa;
	}

	public void setTxtCodEmpresa(MyTextbox txtCodEmpresa) {
		this.txtCodEmpresa = txtCodEmpresa;
	}

	public MyTextbox getTxtCodRuta() {
		return txtCodRuta;
	}

	public void setTxtCodRuta(MyTextbox txtCodRuta) {
		this.txtCodRuta = txtCodRuta;
	}

	public Button getBtnApagar() {
		return btnApagar;
	}

	public void setBtnApagar(Button btnApagar) {
		this.btnApagar = btnApagar;
	}

}
