package com.rjconsultores.ventaboletos.web.gui.controladores.configuracioneccomerciales;

import java.util.HashMap;
import java.util.Map;

import org.apache.commons.lang.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Controller;
import org.zkoss.util.resource.Labels;
import org.zkoss.zhtml.Messagebox;
import org.zkoss.zk.ui.Component;
import org.zkoss.zk.ui.event.Event;
import org.zkoss.zk.ui.event.EventListener;
import org.zkoss.zul.Datebox;
import org.zkoss.zul.Paging;
import org.zkoss.zul.Radiogroup;
import org.zkoss.zul.Textbox;

import com.rjconsultores.ventaboletos.entidad.ClienteCorporativo;
import com.rjconsultores.ventaboletos.entidad.ContratoCorporativo;
import com.rjconsultores.ventaboletos.service.ClienteCorporativoService;
import com.rjconsultores.ventaboletos.service.GrupoContratoService;
import com.rjconsultores.ventaboletos.utilerias.UsuarioLogado;
import com.rjconsultores.ventaboletos.web.utilerias.MyComboboxClienteCorporativo;
import com.rjconsultores.ventaboletos.web.utilerias.MyGenericForwardComposer;
import com.rjconsultores.ventaboletos.web.utilerias.MyListbox;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.HibernateSearchObject;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.PagedListWrapper;
import com.rjconsultores.ventaboletos.web.utilerias.render.RenderContrato;

@Controller("busquedaContratoController")
@Scope("prototype")
public class BusquedaContratoController extends MyGenericForwardComposer {

	private static final String TITULO = "busquedaContratoController.window.title";
	private static Logger log = LogManager.getLogger(BusquedaContratoController.class); 
	private static final long serialVersionUID = 1L;
	@Autowired
	private transient PagedListWrapper<ContratoCorporativo> plwpagingContrato;
	@Autowired	
	private ClienteCorporativoService clienteService;
	@Autowired
	private GrupoContratoService grupoService;
	
	private MyListbox contratoList;
	private Paging pagingContrato;
	private Textbox txtNumContrato;
	private Datebox datInicial;
	private Datebox datFinal;
	private Radiogroup rdgStatus;
	private MyComboboxClienteCorporativo cmbCliente;

	@Override
	public void doAfterCompose(Component comp) throws Exception {
		super.doAfterCompose(comp);

		contratoList.setItemRenderer(new RenderContrato());
		contratoList.addEventListener("onDoubleClick", new EventListener() {

			@Override
			public void onEvent(Event event) throws Exception {
				ContratoCorporativo cc = (ContratoCorporativo) contratoList.getSelected();
				verContrato(cc);
			}
		});
		
		txtNumContrato.focus();
	}

	@SuppressWarnings({ "rawtypes", "unchecked" })
	private void verContrato(ContratoCorporativo gc) {
		if (gc == null) {
			return;
		}

		Map args = new HashMap();
		args.put("contrato", gc);
		args.put("contratoList", contratoList);

		openWindow("/gui/configuraciones_comerciales/negcorporativos/editarContrato.zul",
				Labels.getLabel("editarContratoController.window.title"), args, MODAL);
	}

	private void refreshLista() {
		HibernateSearchObject<ContratoCorporativo> configBusqueda = new HibernateSearchObject<ContratoCorporativo>(ContratoCorporativo.class,pagingContrato.getPageSize());
		boolean filtrado = false;
		
		if (txtNumContrato.getText().length() > 0) {
			filtrado = true;
			configBusqueda.addFilterLike("numContrato", "%" + txtNumContrato.getText().trim().concat("%"));
		}
		
		if (datInicial.getValue() != null ) {
			filtrado = true;
			configBusqueda.addFilterEqual("dataInicial", datInicial.getValue());
		}
		
		if (datFinal.getValue() != null ) {
			filtrado = true;
			configBusqueda.addFilterEqual("dataFinal", datFinal.getValue());
		}
		
		if(rdgStatus.getSelectedItem() != null ) {
			filtrado = true;
			configBusqueda.addFilterEqual("statusContrato", Integer.valueOf(rdgStatus.getSelectedItem().getValue() ));
		}
		
		if (cmbCliente.getSelectedItem() != null ) {
			filtrado = true;
			ClienteCorporativo cliente = (ClienteCorporativo)cmbCliente.getSelectedItem().getValue();
			configBusqueda.addFilterEqual("clienteCorporativoId", cliente.getClienteCorporativoId());
		}
		
		if(UsuarioLogado.isUsuarioAdmContrato()) {
			filtrado = true;
			configBusqueda.addFilterEqual("usuarioRepId", UsuarioLogado.getUsuarioLogado().getUsuarioId());
		}
		
		if(!filtrado) {
			try {
				Messagebox.show(Labels.getLabel("MSG.semFiltro"),
						Labels.getLabel(TITULO),
						Messagebox.OK, Messagebox.INFORMATION);				
			} catch (InterruptedException ex) {
				log.error(ex);
			}
			return;
		}
		
		configBusqueda.addFilterEqual(ACTIVO, Boolean.TRUE);
		configBusqueda.addSortAsc("numContrato");

		plwpagingContrato.init(configBusqueda, contratoList, pagingContrato);

		if (contratoList.getData().length == 0) {
			try {
				Messagebox.show(Labels.getLabel("MSG.ningunRegistro"),
						Labels.getLabel(TITULO),
						Messagebox.OK, Messagebox.INFORMATION);
			} catch (InterruptedException ex) {
				log.error(ex);
			}
		}else {
			preencheComplemento();
		}
	}

	public void onClick$btnPesquisar(Event ev) {
		refreshLista();
	}

	public void onClick$btnRefresh(Event ev) {
		refreshLista();
	}

	public void onClick$btnNovo(Event ev) {
		verContrato(new ContratoCorporativo());
	}
	
	private void preencheComplemento() {
		for (Object item : contratoList.getListData()) {
			ContratoCorporativo obj = (ContratoCorporativo)item;
			if( StringUtils.isEmpty(obj.getNomeClienteCorp()) ) {
				obj.setNomeClienteCorp( clienteService.obtenerID( obj.getClienteCorporativoId()).getNomeClienteCorp() );
			}
			
			if( StringUtils.isEmpty(obj.getNomeGrupoContrato()) ) {
				obj.setNomeGrupoContrato( grupoService.obtenerID( obj.getGrupoContratoId()).getNomeGrupoContrato());
			}
		}		
	}
}
