package com.rjconsultores.ventaboletos.web.gui.controladores.expressos;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.apache.commons.lang.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Controller;
import org.zkoss.util.resource.Labels;
import org.zkoss.zhtml.Messagebox;
import org.zkoss.zk.ui.Component;
import org.zkoss.zk.ui.event.Event;
import org.zkoss.zul.Datebox;
import org.zkoss.zul.Intbox;
import org.zkoss.zul.Paging;

import com.rjconsultores.ventaboletos.entidad.Empresa;
import com.rjconsultores.ventaboletos.entidad.LogAuditoria;
import com.rjconsultores.ventaboletos.service.LogAuditoriaService;
import com.rjconsultores.ventaboletos.utilerias.DateUtil;
import com.rjconsultores.ventaboletos.utilerias.UsuarioLogado;
import com.rjconsultores.ventaboletos.web.utilerias.MyComboboxEstandar;
import com.rjconsultores.ventaboletos.web.utilerias.MyGenericForwardComposer;
import com.rjconsultores.ventaboletos.web.utilerias.MyListbox;
import com.rjconsultores.ventaboletos.web.utilerias.MyTextbox;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.HibernateSearchObject;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.PagedListWrapper;
import com.rjconsultores.ventaboletos.web.utilerias.render.RenderLogAuditoria;
import com.rjconsultores.ventaboletos.web.utilerias.render.RenderLogAuditoriaExpresos;
import com.trg.search.Filter;

import br.com.rjconsultores.auditador.enums.AuditadorTipoAlteracao;

@Controller("logExpresosController")
@Scope("prototype")
public class LogExpresosController extends MyGenericForwardComposer{
	private static final long serialVersionUID = 1L;
	
	@Autowired
	private LogAuditoriaService logAuditoriaService;
	
	@Autowired
	private transient PagedListWrapper<LogAuditoria> plwLogAuditoria;
	
	private static Logger log = LogManager.getLogger(LogExpresosController.class);
	
	private MyListbox logAuditoriaList;
	private Paging pagingLogAuditoria;
	
	private Datebox dtInicio;
	private Datebox dtFim;
	
	private MyTextbox txtCampoAlterado;
	private MyTextbox txtCveUsuario;
	private MyTextbox txtValorNovo;
	private MyTextbox txtValorAnterior;
	private MyTextbox txtIdAuditado;
	
	private Map<String, String> nomeTelas;
	private List<String> lsTela;
	
	@Override
	public void doAfterCompose(Component comp) throws Exception {
		lsTela = recuperarChavesClasse(logAuditoriaService.listarTodasAsTelas());
		
		super.doAfterCompose(comp);
		
		logAuditoriaList.setItemRenderer(new RenderLogAuditoriaExpresos());
	}
	
	private void refreshLista(boolean isGerarRelatorio) throws Exception {
		HibernateSearchObject<LogAuditoria> sistemaBusqueda = new HibernateSearchObject<LogAuditoria>(LogAuditoria.class, pagingLogAuditoria.getPageSize());
		
		Date dataInicio = dtInicio.getValue();
		Date dataFim = dtFim.getValue();
		
		sistemaBusqueda.addFilterGreaterOrEqual("fecmodif", DateUtil.inicioFecha(dataInicio));
		sistemaBusqueda.addFilterLessOrEqual("fecmodif", DateUtil.fimFecha(dataFim));
		
		sistemaBusqueda.addFilterLike("campoAlterado", "%" + "STATUSSOLICITUDEXPRESOID".trim().concat("%"));

		String campoAlterado = txtCampoAlterado.getText();
		if (StringUtils.isNotBlank(campoAlterado)) {
			sistemaBusqueda.addFilterLike("campoAlterado", "%" + campoAlterado.trim().concat("%"));
		}
		
		String valorNovo = txtValorNovo.getText();
		if (StringUtils.isNotBlank(valorNovo)) {
			sistemaBusqueda.addFilterLike("valorNovo", "%" + valorNovo.trim().concat("%"));
		}
		
		String valorAnterior = txtValorAnterior.getText();
		if (StringUtils.isNotBlank(valorAnterior)) {
			sistemaBusqueda.addFilterLike("valorAnterior", "%" + valorAnterior.trim().concat("%"));
		}
		
		String idAuditado = txtIdAuditado.getValue();
		if (StringUtils.isNotBlank(idAuditado)) {
			sistemaBusqueda.addFilterLike("idAuditado", idAuditado + "%");
		}
		
		String cveUsuario = txtCveUsuario.getText();
		if (StringUtils.isNotBlank(cveUsuario)) {
			sistemaBusqueda.addFilterLike("usuario.claveUsuario", "%" + cveUsuario.trim().concat("%"));
		}
		
		sistemaBusqueda.addFilterEqual("tela", "auditarClasse.SolicitudExpreso");

		sistemaBusqueda.addSortAsc("fecmodif");
		sistemaBusqueda.addFilterEqual(ACTIVO, Boolean.TRUE);

		plwLogAuditoria.init(sistemaBusqueda, logAuditoriaList, pagingLogAuditoria);
		
		if (logAuditoriaList.getData().length == 0) {
			if(isGerarRelatorio){
				throw new Exception(Labels.getLabel("MSG.ningunRegistroRelatorio"));
			}else{
				try {
					Messagebox.show(Labels.getLabel("MSG.ningunRegistro"),Labels.getLabel("busquedaLogAuditoriaController.window.title"), Messagebox.OK, Messagebox.INFORMATION);
				} catch (InterruptedException ex) {
					log.error("", ex);
				}
			}
		}
	}
	
	private List<String> recuperarChavesClasse(List<String> lsTela) {
		
		nomeTelas = new HashMap<String, String>();
		List<String> lsTelasAux = new ArrayList<String>();
		
		for (String tela : lsTela) {
			lsTelasAux.add(Labels.getLabel(tela, tela));
			nomeTelas.put(tela, Labels.getLabel(tela, tela));
		}
		Collections.sort(lsTelasAux);
		return lsTelasAux;
	}
	
	private String recuperarChaveNomeTela(String tela) {
		
		String chave = null;
		for (Entry<String, String> entry : nomeTelas.entrySet()) {
            if (entry.getValue().equals(tela)) {
            	chave =  entry.getKey();
            }
        }
		return chave;
	}
	
	public void onClick$btnPesquisa(Event ev) throws Exception {
		refreshLista(false);
	}
}
